@@ -25,12 +25,14 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
+#include <gspell/gspell.h>
+
 #include "gr-details-page.h"
 #include "gr-recipe-store.h"
 #include "gr-app.h"
 #include "gr-window.h"
 #include "gr-utils.h"
-#include "gr-image-editor.h"
+#include "gr-images.h"
 #include "gr-image-viewer.h"
 #include "gr-ingredients-list.h"
 #include "gr-timer.h"
@@ -55,28 +57,31 @@ timer_complete (GrTimer *timer)
         g_autofree char *action = NULL;
         GrRecipeStore *store;
         g_autoptr(GrRecipe) recipe = NULL;
+        const char *id;
         const char *name;
 
         app = g_application_get_default ();
 
-        name = gr_timer_get_name (timer);
+        id = gr_timer_get_name (timer);
+
+        store = gr_app_get_recipe_store (GR_APP (app));
+        recipe = gr_recipe_store_get_recipe (store, id);
+        name = gr_recipe_get_name (recipe);
 
         body = g_strdup_printf (_("Your cooking timer for “%s” has expired."), name);
 
         notification = g_notification_new (_("Time is up!"));
         g_notification_set_body (notification, body);
-        action = g_strdup_printf ("app.timer-expired::%s", name);
+        action = g_strdup_printf ("app.timer-expired::%s", id);
         g_notification_set_default_action (notification, action);
 
         g_application_send_notification (app, "timer", notification);
 
-        store = gr_app_get_recipe_store (GR_APP (app));
-        recipe = gr_recipe_store_get_recipe (store, name);
         gr_recipe_store_add_cooked (store, recipe);
 }
 
 static CookingData *
-cooking_data_new (const char *name)
+cooking_data_new (const char *id)
 {
         CookingData *cd;
 
@@ -84,7 +89,7 @@ cooking_data_new (const char *name)
         cd->ingredients = FALSE;
         cd->preheat = FALSE;
         cd->instructions = FALSE;
-        cd->timer = gr_timer_new (name);
+        cd->timer = gr_timer_new (id);
 
         g_signal_connect (cd->timer, "complete", G_CALLBACK (timer_complete), NULL);
 
@@ -119,7 +124,7 @@ struct _GrDetailsPage
         GtkWidget *warning_box;
         GtkWidget *spicy_warning;
         GtkWidget *garlic_warning;
-        GtkWidget *ingredients_list;
+        GtkWidget *ingredients_box;
         GtkWidget *instructions_label;
         GtkWidget *cooking_revealer;
         GtkWidget *ingredients_check;
@@ -134,6 +139,12 @@ struct _GrDetailsPage
         GtkWidget *duration_stack;
         GtkWidget *remaining_time_label;
         GtkWidget *chef_label;
+        GtkWidget *edit_button;
+        GtkWidget *delete_button;
+        GtkWidget *notes_field;
+        GtkWidget *description_label;
+
+        guint save_timeout;
 };
 
 G_DEFINE_TYPE (GrDetailsPage, gr_details_page, GTK_TYPE_BOX)
@@ -145,7 +156,7 @@ timer_active_changed (GrTimer       *timer,
                       GParamSpec    *pspec,
                       GrDetailsPage *page)
 {
-        if (strcmp (gr_timer_get_name (timer), gr_recipe_get_name (page->recipe)) != 0)
+        if (strcmp (gr_timer_get_name (timer), gr_recipe_get_id (page->recipe)) != 0)
                 return;
 
         if (gr_timer_get_active (timer)) {
@@ -187,17 +198,17 @@ static void
 set_cooking (GrDetailsPage *page,
              gboolean       cooking)
 {
-        const char *name;
+        const char *id;
         CookingData *cd;
 
-        name = gr_recipe_get_name (page->recipe);
+        id = gr_recipe_get_id (page->recipe);
 
-        cd = g_hash_table_lookup (page->cooking, name);
+        cd = g_hash_table_lookup (page->cooking, id);
 
         if (cooking) {
                 if (!cd) {
-                        cd = cooking_data_new (name);
-                        g_hash_table_insert (page->cooking, g_strdup (name), cd);
+                        cd = cooking_data_new (id);
+                        g_hash_table_insert (page->cooking, g_strdup (id), cd);
                 }
 
                 g_object_set (page->ingredients_check, "active", cd->ingredients, NULL);
@@ -214,7 +225,7 @@ set_cooking (GrDetailsPage *page,
                 if (cd) {
                         g_signal_handlers_disconnect_by_func (cd->timer, G_CALLBACK (timer_active_changed), page);
                         g_signal_handlers_disconnect_by_func (cd->timer, G_CALLBACK (timer_remaining_changed), page);
-                        g_hash_table_remove (page->cooking, name);
+                        g_hash_table_remove (page->cooking, id);
                 }
 
                 g_object_set (page->timer, "timer", NULL, NULL);
@@ -304,12 +315,12 @@ serves_value_changed (GrDetailsPage *page)
 static void
 start_or_stop_timer (GrDetailsPage *page)
 {
-        const char *name;
+        const char *id;
         CookingData *cd;
 
-        name = gr_recipe_get_name (page->recipe);
+        id = gr_recipe_get_id (page->recipe);
 
-        cd = g_hash_table_lookup (page->cooking, name);
+        cd = g_hash_table_lookup (page->cooking, id);
 
         g_assert (cd && cd->timer);
 
@@ -440,11 +451,11 @@ check_clicked (GtkWidget     *button,
                GrDetailsPage *page)
 {
         CookingData *cd;
-        const char *name;
+        const char *id;
         gboolean active;
 
-        name = gr_recipe_get_name (page->recipe);
-        cd = g_hash_table_lookup (page->cooking, name);
+        id = gr_recipe_get_id (page->recipe);
+        cd = g_hash_table_lookup (page->cooking, id);
 
         g_assert (cd);
 
@@ -482,6 +493,11 @@ details_page_finalize (GObject *object)
         g_clear_object (&self->exporter);
         g_clear_pointer (&self->cooking, g_hash_table_unref);
 
+        if (self->save_timeout) {
+                g_source_remove (self->save_timeout);
+                self->save_timeout = 0;
+        }
+
         G_OBJECT_CLASS (gr_details_page_parent_class)->finalize (object);
 }
 
@@ -514,9 +530,58 @@ all_headers (GtkListBoxRow *row,
         gtk_list_box_row_set_header (row, header);
 }
 
+static gboolean
+save_notes (gpointer data)
+{
+        GrDetailsPage *page = data;
+        GtkTextBuffer *buffer;
+        GtkTextIter start, end;
+        g_autofree char *text = NULL;
+        GrRecipeStore *store;
+        g_autofree char *id = NULL;
+        g_autofree char *notes = NULL;
+        g_autoptr(GError) error = NULL;
+
+        buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (page->notes_field));
+        gtk_text_buffer_get_bounds (buffer, &start, &end);
+        text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
+
+        g_object_get (page->recipe,
+                      "id", &id,
+                      "notes", &notes,
+                      NULL);
+        if (g_strcmp0 (notes, text) == 0)
+                goto out;
+
+        g_object_set (page->recipe, "notes", text, NULL);
+
+        store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
+        if (!gr_recipe_store_update_recipe (store, page->recipe, id, &error)) {
+                g_print ("Error: %s\n", error->message);
+        }
+
+out:
+        page->save_timeout = 0;
+
+        return G_SOURCE_REMOVE;
+}
+
+static void
+schedule_save (GtkTextBuffer *buffer, GrDetailsPage *page)
+{
+        if (page->save_timeout) {
+                g_source_remove (page->save_timeout);
+                page->save_timeout = 0;
+        }
+
+        page->save_timeout = g_timeout_add (250, save_notes, page);
+}
+
 static void
 gr_details_page_init (GrDetailsPage *page)
 {
+        GspellTextView *gspell_view;
+
         gtk_widget_set_has_window (GTK_WIDGET (page), FALSE);
         gtk_widget_init_template (GTK_WIDGET (page));
         connect_store_signals (page);
@@ -530,8 +595,10 @@ gr_details_page_init (GrDetailsPage *page)
                                      NULL,
                                      NULL);
 
-        gtk_list_box_set_header_func (GTK_LIST_BOX (page->ingredients_list),
-                                      all_headers, NULL, NULL);
+        g_signal_connect (gtk_text_view_get_buffer (GTK_TEXT_VIEW (page->notes_field)), "changed", G_CALLBACK (schedule_save), page);
+
+        gspell_view = gspell_text_view_get_from_gtk_text_view (GTK_TEXT_VIEW (page->notes_field));
+        gspell_text_view_basic_setup (gspell_view);
 }
 
 static void
@@ -551,7 +618,7 @@ gr_details_page_class_init (GrDetailsPageClass *klass)
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, warning_box);
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, spicy_warning);
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, garlic_warning);
-        gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, ingredients_list);
+        gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, ingredients_box);
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, instructions_label);
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, cooking_revealer);
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, ingredients_check);
@@ -566,6 +633,10 @@ gr_details_page_class_init (GrDetailsPageClass *klass)
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, duration_stack);
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, remaining_time_label);
         gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, chef_label);
+        gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, edit_button);
+        gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, delete_button);
+        gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, notes_field);
+        gtk_widget_class_bind_template_child (widget_class, GrDetailsPage, description_label);
 
         gtk_widget_class_bind_template_callback (widget_class, edit_recipe);
         gtk_widget_class_bind_template_callback (widget_class, delete_recipe);
@@ -596,43 +667,64 @@ populate_ingredients (GrDetailsPage *page,
                       int            denom)
 {
         g_autoptr(GtkSizeGroup) group = NULL;
+        g_autofree char **segments = NULL;
         g_auto(GStrv) ings = NULL;
-        int i;
+        int i, j;
+        GtkWidget *list;
+        GtkWidget *label;
+
+        container_remove_all (GTK_CONTAINER (page->ingredients_box));
 
         group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-        container_remove_all (GTK_CONTAINER (page->ingredients_list));
-        ings = gr_ingredients_list_get_ingredients (page->ingredients);
-        for (i = 0; ings[i]; i++) {
-                GtkWidget *row;
-                GtkWidget *box;
-                GtkWidget *label;
-                g_autofree char *s = NULL;
+        segments = gr_ingredients_list_get_segments (page->ingredients);
+        for (j = 0; segments[j]; j++) {
+                if (segments[j] && segments[j][0]) {
+                        label = gtk_label_new (segments[j]);
+                        gtk_widget_show (label);
+                        gtk_label_set_xalign (GTK_LABEL (label), 0);
+                        gtk_style_context_add_class (gtk_widget_get_style_context (label), "heading");
+                        gtk_container_add (GTK_CONTAINER (page->ingredients_box), label);
+                }
 
-                box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-                gtk_widget_show (box);
+                list = gtk_list_box_new ();
+                gtk_widget_show (list);
+                gtk_style_context_add_class (gtk_widget_get_style_context (list), "frame");
+                gtk_list_box_set_selection_mode (GTK_LIST_BOX (list), GTK_SELECTION_NONE);
+                gtk_list_box_set_header_func (GTK_LIST_BOX (list), all_headers, NULL, NULL);
+                gtk_container_add (GTK_CONTAINER (page->ingredients_box), list);
 
-                s = gr_ingredients_list_scale_unit (page->ingredients, ings[i], num, denom);
-                label = gtk_label_new (s);
-                g_object_set (label,
-                              "visible", TRUE,
-                              "xalign", 0.0,
+                ings = gr_ingredients_list_get_ingredients (page->ingredients, segments[j]);
+                for (i = 0; ings[i]; i++) {
+                        GtkWidget *row;
+                        GtkWidget *box;
+                        g_autofree char *s = NULL;
+
+                        box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+                        gtk_widget_show (box);
+
+                        s = gr_ingredients_list_scale_unit (page->ingredients, segments[j], ings[i], num, denom);
+                        label = gtk_label_new (s);
+                        g_object_set (label,
+                                      "visible", TRUE,
+                                      "xalign", 0.0,
+                                      "margin", 10,
+                                      NULL);
+                        gtk_style_context_add_class (gtk_widget_get_style_context (label), "dim-label");
+                        gtk_container_add (GTK_CONTAINER (box), label);
+                        gtk_size_group_add_widget (group, label);
+
+                        label = gtk_label_new (ings[i]);
+                        g_object_set (label,
+                                      "visible", TRUE,
+                                      "xalign", 0.0,
                               "margin", 10,
                               NULL);
-                gtk_style_context_add_class (gtk_widget_get_style_context (label), "dim-label");
-                gtk_container_add (GTK_CONTAINER (box), label);
-                gtk_size_group_add_widget (group, label);
+                        gtk_container_add (GTK_CONTAINER (box), label);
 
-                label = gtk_label_new (ings[i]);
-                g_object_set (label,
-                              "visible", TRUE,
-                              "xalign", 0.0,
-                              "margin", 10,
-                              NULL);
-                gtk_container_add (GTK_CONTAINER (box), label);
-
-                gtk_container_add (GTK_CONTAINER (page->ingredients_list), box);
-                row = gtk_widget_get_parent (box);
-                gtk_list_box_row_set_activatable (GTK_LIST_BOX_ROW (row), FALSE);
+                        gtk_container_add (GTK_CONTAINER (list), box);
+                        row = gtk_widget_get_parent (box);
+                        gtk_list_box_row_set_activatable (GTK_LIST_BOX_ROW (row), FALSE);
+                }
         }
 
         gtk_widget_hide (page->warning_box);
@@ -664,13 +756,15 @@ void
 gr_details_page_set_recipe (GrDetailsPage *page,
                             GrRecipe      *recipe)
 {
-        const char *name;
+        const char *id;
         const char *author;
         const char *prep_time;
         const char *cook_time;
         int serves;
         const char *ingredients;
         const char *instructions;
+        const char *notes;
+        const char *description;
         g_autoptr(GdkPixbuf) pixbuf = NULL;
         GrRecipeStore *store;
         g_autoptr(GrChef) chef = NULL;
@@ -681,13 +775,15 @@ gr_details_page_set_recipe (GrDetailsPage *page,
 
         g_set_object (&page->recipe, recipe);
 
-        name = gr_recipe_get_name (recipe);
+        id = gr_recipe_get_id (recipe);
         author = gr_recipe_get_author (recipe);
         serves = gr_recipe_get_serves (recipe);
         prep_time = gr_recipe_get_prep_time (recipe);
         cook_time = gr_recipe_get_cook_time (recipe);
         ingredients = gr_recipe_get_ingredients (recipe);
         instructions = gr_recipe_get_instructions (recipe);
+        notes = gr_recipe_get_notes (recipe);
+        description = gr_recipe_get_description (recipe);
 
         g_object_get (recipe, "images", &images, NULL);
         gr_image_viewer_set_images (GR_IMAGE_VIEWER (page->recipe_image), images);
@@ -704,7 +800,7 @@ gr_details_page_set_recipe (GrDetailsPage *page,
         gtk_spin_button_set_value (GTK_SPIN_BUTTON (page->serves_spin), serves);
         gtk_widget_set_sensitive (page->serves_spin, ing != NULL);
 
-        cooking = g_hash_table_lookup (page->cooking, name) != NULL;
+        cooking = g_hash_table_lookup (page->cooking, id) != NULL;
         set_cooking (page, cooking);
 
         store = gr_app_get_recipe_store (GR_APP (g_application_get_default ()));
@@ -727,6 +823,26 @@ gr_details_page_set_recipe (GrDetailsPage *page,
         else {
                 gtk_widget_hide (page->chef_label);
         }
+
+        gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (page->notes_field)),
+                                  notes ? notes : "", -1);
+
+        if (description && description[0]) {
+                gtk_label_set_label (GTK_LABEL (page->description_label), description);
+                gtk_widget_show (page->description_label);
+        }
+        else {
+                gtk_widget_hide (page->description_label);
+        }
+
+        if (gr_recipe_is_readonly (recipe)) {
+                gtk_widget_hide (page->edit_button);
+                gtk_widget_hide (page->delete_button);
+        }
+        else {
+                gtk_widget_show (page->edit_button);
+                gtk_widget_show (page->delete_button);
+        }
 }
 
 static void
