@@ -28,9 +28,14 @@ G_BEGIN_DECLS
 
 G_DECLARE_FINAL_TYPE (GrImageViewer, gr_image_viewer, GR, IMAGE_VIEWER, GtkBox)
 
-GrImageViewer *gr_image_viewer_new        (void);
-void           gr_image_viewer_set_images (GrImageViewer *viewer,
-                                           GArray        *images);
+GrImageViewer *gr_image_viewer_new          (void);
+void           gr_image_viewer_set_images   (GrImageViewer *viewer,
+                                             GArray        *images);
+
+void           gr_image_viewer_add_image    (GrImageViewer *viewer);
+void           gr_image_viewer_remove_image (GrImageViewer *viewer);
+void           gr_image_viewer_rotate_image (GrImageViewer *viewer,
+                                             int            angle);
 
 G_END_DECLS
 
